/** Privacy-sensitive analytics 
 * 
 * See function gatherInfo() to see exactly what information is collected
 * 
 * 
 */
function gatherInfo(){
    var tz = new Date().getTimezoneOffset();
    
    var referrer_domain = '';
    var referrer = document.referrer;
    if (referrer.startsWith(document.location.protocol + "//" + document.location.hostname)){
        // In-site navigation, blank the referrer - we're not looking to stalk users around the site
        referrer = '';
    }
    
    if (referrer.length > 8){
        referrer_domain = new URL(document.referrer).hostname;   
    }
    
    return {
        domain : window.location.hostname,
        page: window.location.pathname,
        referrer: referrer,
        referrer_domain: referrer_domain,
        platform: navigator.platform,
        timezone: tz,
        sess_id: getSessionId(),
        state: "PageView",
        responseTime: window.performance.timing.domContentLoadedEventStart - window.performance.timing.navigationStart
    }
}

function getSessionId(){
    // Only if enabled
    if (!window.analytics_gen_psuedoid){
        return "none";
    }

    // Every so often, we want to rotate the session ID to mitigate its impact
    var n = randomInteger(1, 10);
    
    var k = window.location.hostname + "_sess-id";
    var i = sessionStorage.getItem(k);
    
    // If there's no item in storage, or if we've decided to rotate
    if (!i || n == 5){
        i = createUUID();
        sessionStorage.setItem(k,i);
    }
    
    return i;
}

function createUUID() {
    // Create a short UUID
    return 'xxxx-xxxx'.replace(/[xy]/g, function(c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
    });
}

function submit(info){
    var xml = new XMLHttpRequest();
    xml.onreadystatechange = function () {}; // Do nothing
    
    // If we're being viewed on a .onion, use the .onion instead
    // (keeps everything within the tor network)
    if (window.location.hostname.toLowerCase().endsWith(".onion")){
        xml.open("POST", window.analytics_endpoint_onion + "/write");
    }else{
        xml.open("POST", window.analytics_endpoint);
    }
    xml.setRequestHeader("Content-Type", "application/json");
    xml.send(JSON.stringify(info));
}

function collect_and_submit(){
    if (window.analytics_active && document.cookie.indexOf('analytics_optout=')){
        ping();
        console.log("gathering");
        submit(gatherInfo())
    }
}

function doDocumentReady(fn){
    // $(document).ready() without jquery
    if (document.readyState === "complete" ||
        (document.readyState !== "loading" && !document.documentElement.doScroll)){
            fn();
    } else {
            document.addEventListener("DOMContentLoaded", fn);
    }
}

// All this does is place a request to Tor the endpoint
// its only purpose is to ensure we have a circuit open ready for use during write
// so that writes can complete faster
function ping(){
    if (window.location.hostname.toLowerCase().endsWith(".onion")){
        var xml = new XMLHttpRequest();
        xml.onreadystatechange = function () {}; // Do nothing        
        xml.open("GET", window.analytics_endpoint_onion + "/ping");
        xml.send();
    }
}

function randomInteger(min, max) {
  return Math.floor(Math.random() * (max - min + 1)) + min;
}


/* These can be overridden in the base document */
window.analytics_active = true;
window.analytics_gen_psuedoid = true;
window.analytics_endpoint = 'https://pfanalytics.bentasker.co.uk/write';
window.analytics_endpoint_onion = 'http://wftlf4ke7xwqkjrrju4aok57pkr7kfks5t5uqmipxlyjucwjycdql7id.onion';
doDocumentReady(collect_and_submit);
