#!/usr/bin/env python3
#
#

import influxdb_client
from influxdb_client.client.write_api import SYNCHRONOUS


DEST_MEASUREMENT="pf_analytics"
DEST_BUCKET="websites/analytics"
DEST_ORG=""


def query_and_write_agg_info(q, key, query_api, write_api, field_pref=''):
    ''' Run one of the map queries, pull out results and write the resulting aggregates back in
    
    '''
    print(q)
    tables = query_api.query(q)
    for table in tables:
        print(table)
        for row in table.records:
            print (row.values)
            p = influxdb_client.Point(DEST_MEASUREMENT)
            p.tag(key, row.values[key])
            p.tag("domain", row.values['domain'])
            p.tag("host", row.values['host'])
            if "action" in row.values:
                p.tag("action", row.values['action'])
            p.tag("page", row.values['page'])
            p.field(f"{field_pref}PageViews", row.values['_value'])
            p.time(row.values['_time'])
            write_api.write(DEST_BUCKET, DEST_ORG, p)


def query_and_write_field_val(q, key, query_api, write_api):
    ''' Run one of the map queries, pull out results and write the resulting aggregates back in
    
    '''
    print(q)
    tables = query_api.query(q)
    for table in tables:
        print(table)
        for row in table.records:
            print (row.values)
            p = influxdb_client.Point(DEST_MEASUREMENT)
            p.tag("domain", row.values['domain'])
            p.tag("host", row.values['host'])
            p.tag("page", row.values['page'])
            if "action" in row.values:
                p.tag("action", row.values['action'])    
            p.tag("page", row.values['page'])                
            p.field(key, row.values['_value'])
            p.time(row.values['_time'])
            write_api.write(DEST_BUCKET, DEST_ORG, p)

# Setup
client = influxdb_client.InfluxDBClient(url="http://telegrafexternal.bentasker.co.uk", token="", org="")
write_api = client.write_api(write_options=SYNCHRONOUS)
query_api = client.query_api()

START="-3h"

# Pull platform info
q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test")
  |> filter(fn: (r) => r._field == "platform")
  |> map(fn: (r) => ({ r with _field: r._value}))
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with platform: r._field, _field: "viewcount"})) 
'''
query_and_write_agg_info(q,"platform", query_api, write_api, 'platform_')


        
# Pull Referrering domain info        
q = '''  
import "strings"

from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test")
  |> filter(fn: (r) => r._field == "referrer_domain")
  |> filter(fn: (r) => r._value != "")
  |> map(fn: (r) => ({ r with _field: 
                       if (strings.hasPrefix(v: r._value, prefix: "www.google")) then "google" 
                       else r._value}))
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with referringdomain: r._field, _field: "viewcount"}))  
'''
query_and_write_agg_info(q,"referringdomain", query_api, write_api, 'referrer_')


q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test")
  |> filter(fn: (r) => r._field == "timezone")


  |> map(fn: (r) => ({ r with _field: string(v: r._value)}))
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with timezone: r._field, _field: "tz_pageviews"})) 
'''
query_and_write_agg_info(q,"timezone", query_api, write_api, 'tz_')


q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: mean)
  |> group()

'''
query_and_write_field_val(q, "response_time", query_api, write_api)

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: max)
  |> group()

'''
query_and_write_field_val(q, "max_response_time", query_api, write_api)

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: max)
  |> group()

'''
query_and_write_field_val(q, "min_response_time", query_api, write_api)


q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> group()

'''
query_and_write_field_val(q, "requests", query_api, write_api)
